<?php

namespace App\Factory;

use App\Exception;

class ProductFactory {

    private $connection;
    private $util;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function getProductMap ($product) {
        $sql = "
            SELECT CDPRODUTO 
              FROM PRODEXTERNOINTE
              WHERE CDPRODINTE = :CDPRODEXTERNO
                AND CDFILIAL IS NULL
                AND CDPROJETO  = (SELECT CDPROJETO
                                   FROM (SELECT CDPROJETO
                                           FROM CADASTROPROJETO
                                          WHERE UPPER(DSPROJETO) = UPPER('RAPPI'))
                                WHERE ROWNUM = 1)
        ";

        $params = [
            ':CDPRODEXTERNO' => $product
        ];
        
        $extProduct = $this->connection->fetchAssoc($sql, $params);

        if(empty($extProduct))
            return null;
        
        return $this->getProduct($extProduct['CDPRODUTO']);
    }
    
    public function getProduct ($product) {
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDARVPROD  = :CDARVPROD
                OR CDPRODINTE = :CDPRODINTE
                OR CDPRODUTO  = :CDPRODUTO
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);

        $params = [
            ":CDARVPROD"  => $productCode,
            ':CDPRODINTE' => $productCode,
            ':CDPRODUTO'  => $productCode
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertProduct ($product) {
        $sql = "
            INSERT INTO ITCOMANDAVEN
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
    			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
    			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
    			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
    			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, NRINSCRCONS, DSOBSPEDDIGCMD)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
    			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
    			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'),
    			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
    			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, :NRINSCRCONS, :DSOBSPEDDIGCMD)
    	";
    		
    	$params = [
    	    ":CDFILIAL"       => $product["CDFILIAL"],
			":NRVENDAREST"    => $product["NRVENDAREST"],
			":NRCOMANDA"      => $product["NRCOMANDA"],
			":NRPRODCOMVEN"   => $product["NRPRODCOMVEN"],
			":CDPRODUTO"      => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODUTO"]),
			":QTPRODCOMVEN"   => $product["QTPRODCOMVEN"],
			":VRPRECCOMVEN"   => $this->util->toFloat($product["VRPRECCOMVEN"]),
			":TXPRODCOMVEN"   => $product["TXPRODCOMVEN"],
			":IDSTPRCOMVEN"   => $product["IDSTPRCOMVEN"],
			":VRDESCCOMVEN"   => $this->util->toFloat($product["VRDESCCOMVEN"]),
			":NRLUGARMESA"    => $product["NRLUGARMESA"],
			":CURRDATE"       => date("d/m/Y H:i:s"),
			":IDPRODIMPFIS"   => $product["IDPRODIMPFIS"],
			":CDLOJA"         => $product["CDLOJA"],
			":VRACRCOMVEN"    => $this->util->toFloat($product["VRACRCOMVEN"]),
			":NRSEQPRODCOM"   => $product["NRSEQPRODCOM"],
			":NRSEQPRODCUP"   => $product["NRSEQPRODCUP"],
			":VRPRECCLCOMVEN" => $this->util->toFloat($product["VRPRECCLCOMVEN"]),
			":CDCAIXACOLETOR" => $product["CDCAIXACOLETOR"],
			":CDPRODPROMOCAO" => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODPROMOCAO"]),
			":CDVENDEDOR"     => $product["CDVENDEDOR"],
			":NRINSCRCONS"    => $product["NRINSCRCONS"],
			":DSOBSPEDDIGCMD" => $product["TXPRODCOMVEN"]
    	];
    	
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getObservation ($product, $branch, $store) {
    	$sql = "
    		SELECT *
    		  FROM OCORRENCIA
    		 WHERE CDPRODUTO     = :CDPRODUTO
    		   AND IDCONTROLAOBS = 'R'
    		   AND CDGRPOCOR = (SELECT CDGRPOCORPED FROM LOJA
                                    WHERE CDFILIAL = :CDFILIAL
                                      AND CDLOJA = :CDLOJA)
    	";
    	
    	$params = [
    		":CDPRODUTO" => $product,
    		":CDFILIAL" => $branch,
    		":CDLOJA" => $store
    	];
    	
    	$obs = $this->connection->fetchAll($sql, $params);
    	if(count($obs) > 1) {
            throw new Exception\ParamsException("O produto {$product} está vinculado a mais de uma ocorrência no sistema.");
        }
        
        return !empty($obs) ? $obs[0] : null;
    }
    
    public function insertObservation ($obs) {
        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)";
		
	    $params = [
	        ":CDFILIAL"        => $obs["CDFILIAL"],
            ":NRVENDAREST"     => $obs["NRVENDAREST"],
            ":NRCOMANDA"       => $obs["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $obs["NRPRODCOMVEN"],
            ":NRPRODCOMVENOBS" => $obs["NRPRODCOMVENOBS"],
            ":CDOCORR"         => $obs["CDOCORR"],
            ":CDGRPOCOR"       => $obs["CDGRPOCOR"],
            ":QTPRODCOMVENOBS" => $obs["QTPRODCOMVENOBS"]  
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getIdTipCobra ($branch, $store) {
        $sql = "
            SELECT IDTIPCOBRA
               FROM LOJA
            WHERE CDFILIAL = :CDFILIAL
               AND CDLOJA  = :CDLOJA
        ";
        
        $params = [
            ":CDFILIAL" => $branch,
            ":CDLOJA"   => $store
        ];
        
        return $this->connection->executeQuery($sql, $params)[0]["IDTIPCOBRA"];
    }
     public function getOcorencia ($observation, $isAcr = false) {
    	if($isAcr){
    	    $sql = "
        		SELECT *
        		FROM OCORRENCIA
        		WHERE CDGRPOCOR = :CDGRPOCOR
        		  AND CDOCORR   = :CDOCORR
        		  AND IDCONTROLAOBS = 'A'
        		  AND CDPRODUTO IS NOT NULL";
    	} else{
    	    $sql = "
        		SELECT *
        		FROM OCORRENCIA
        		WHERE CDGRPOCOR = :CDGRPOCOR
        		  AND CDOCORR   = :CDOCORR
        		  AND IDCONTROLAOBS IN ('O', 'R')";
    	}
    	
    	$obs = explode("|", $observation);
    	
    	$params = [
    		":CDGRPOCOR" => $obs[0],
    		":CDOCORR"   => $obs[1]
    	];
    	
    	return $this->connection->fetchAssoc($sql, $params);
    }
}