<?php

namespace App\Util;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;
use App\Exception;

class Util {
    
    private $connection;
    private $accessToken;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function setAccessToken ($accessToken) {
        $this->accessToken = $accessToken;
    }
    
    public function getAccessToken () {
        return $this->accessToken;
    }
    
    public function getBearerToken($httpAuth) {
        if(!is_array($httpAuth))
            return 0;

        foreach($httpAuth as $authItem) {
            if(strpos($authItem, "bearer") !== false)  {
                return explode("bearer", $authItem)[1];
            }                       
        }
        
        return null;
    }
    
    public function isValidToken($token) {
        return (trim($token) == "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E");
    }
    
    public function newCode($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }
    
    public function executeWebService ($service_url, $method, $custom_params = "") {
        /* Definição do header */
        $header = [
            "Content-type: application/json",
        ];

        if (!empty($this->getAccessToken()))
            array_push($header, "x-authorization: Bearer " . $this->getAccessToken());

        /* Configuração do cURL para a execução do web service */
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $service_url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $custom_params);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_FAILONERROR, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        
        // curl_setopt($curl, CURLOPT_PROXY, "http://192.168.122.121:3128");
        
        /* Execução do web service */
        $curl_response = curl_exec($curl);
        $httpCode      = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        
        //invalid access token
        if($httpCode == 401) {
            $this->writeLog("execution", "", date("d/m/Y H:i:s")." Sem autorização para comunicação com a Rappi\n");
            $this->resetAccessToken();
        }
        
        if (!empty(curl_error($curl)) || (stripos($curl_response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $service_url,
                'method'    => $method,
                'params'    => $custom_params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curl_response
            ]));
            
            curl_close($curl);
            throw new Exception\NetworkException("Erro na comunicação com a API da Rappi");
        }
        
        curl_close($curl);
        return $curl_response;
    }
    
    public function authorizeAccess ($targetUrl, $config, $audience) {
        
        $this->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando nova autenticação: ");

        $curl = curl_init();
        $params = json_encode([
            "client_id" => $config["IDTOKENINTGERAL"],
            "client_secret" => $config["IDSENHATOKENGERAL"],
            "audience" => $audience,
            "grant_type" => "client_credentials"
        ]);

        curl_setopt($curl, CURLOPT_URL, $targetUrl);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);    
        curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_FAILONERROR, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        
        $curl_response = curl_exec($curl);
        $httpCode      = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $access = json_decode($curl_response, true);
        $fileName      = date("d") . ".txt";
        
        if (empty($access)) {
            $this->writeLog("execution", "", "Erro\n");

            $this->writeLog(
                'network', 
                $fileName, 
                $this->getContentAuthLog(['Content-Type: application/json'], $targetUrl, $params, $httpCode, curl_error($curl), $curl_response)
            );
            
            curl_close($curl);
            throw new \Exception("Não foi possível recuperar o token de acesso à API. Sem resposta do endpoint.");
        } 
        
        if(!isset($access["access_token"])){
            $this->writeLog("execution", "", "Erro\n");

            $this->writeLog(
                'network', 
                $fileName, 
                $this->getContentAuthLog(['Content-Type: application/json'], $targetUrl, $params, $httpCode, curl_error($curl), $curl_response)
            );
            
            curl_close($curl);
            throw new \Exception("Não foi possível recuperar o token de acesso à API. Erro de credencial.");        
        }
        
        $this->writeLog("execution", "", "Sucesso\n");        
        $this->setAccessToken($access["access_token"]);
        curl_close($curl);
    }
    
    public function executeGoogleService($url, $method, $header, $params) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        // curl_setopt($ch, CURLOPT_PROXY, "http://192.168.122.121:3128");
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($ch)) || (stripos($response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => $method,
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));

            if(strpos($url, 'google') !== false) throw new Exception\NetworkException("Erro na comunicação com a Google.");
        }
        curl_close($ch);

        return $response;
    }
    
    private function getContentAuthLog($header, $targetUrl, $params, $httpCode, $error, $curl_response) {
        return $content = $this->formatNetworkError([
            'eventTime' => date("d/m/Y H:i:s"),
            'header'    => $header,
            'url'       => $targetUrl,
            'method'    => "POST",
            'params'    => $params,
            'httpCode'  => $httpCode,
            'error'     => $error,
            'response'  => $curl_response
        ]);
    }
    
    private function filterHeader ($output) {
        $headers = [];
        $data = explode("\n",$output);
        array_shift($data);
        
        foreach($data as $part){
        
            //some headers will contain ":" character (Location for example), and the part after ":" will be lost, Thanks to @Emanuele
            $middle = explode(":", $part, 2);
        
            //Supress warning message if $middle[1] does not exist, Thanks to @crayons
            if ( !isset($middle[1]) ) { $middle[1] = null; }
        
            $headers[trim($middle[0])] = trim($middle[1]);
        }
        
        return isset($headers["X-Auth-Int"]) ? ["token" => $headers["X-Auth-Int"]] : null;
    }
    
    public function writeLog($type, $fileName, $content) {
        
        $configsFile  = file_get_contents(__DIR__ . "/../../settings/configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $config['logPath'] . "/$clientFolder";
        
        switch($type) {
            case 'order':
                $folder = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $this->putContents($type, $folder, $fileName, $content);

                break;
                
            case 'order_processed':
                $folder = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/";
                $currentFile = "$folder" . date("d") . ".txt";
                if(!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                
                file_put_contents($currentFile, $content, FILE_APPEND);
                break;
                
            case 'order_error':
                $folder = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName . ".txt";
                if(!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                

                file_put_contents($currentFile, $content, FILE_APPEND);
                break;
                
            case 'network':
                $folder = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";

                if(!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                
                $this->putContents($type, $folder, $fileName, $content);
                break;
                
            case 'execution':
                $folder   = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
                $currentFile = "$folder" . date("d") . ".txt";
                if(!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                
                file_put_contents($currentFile, $content, FILE_APPEND);
                break;
        }
    }
    
    private function putContents($type, $folder, $fileName, $content) {
        $currentFile = "$folder/$fileName";
        
        if(!file_exists($folder)) 
            mkdir($folder, 0775, true);

        if($type !== 'network')
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
            
        else 
            file_put_contents($currentFile, $content);
    }
    
    private function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }
    
    public function sendMail($recipient, $cc, $subject, $body, $params) {
        try {
            $configFile = file_get_contents(__DIR__ . "/../../settings/configs/configs.json");
            $config     = json_decode($configFile, true);
        
            $mail = new PHPMailer(true);

            $mail->isSMTP();                            
            $mail->Host       = $config["host"];          
            $mail->SMTPAuth   = true;                   
            $mail->Username   = $config["username"];
            $mail->Password   = $this->decrypt($config["password"]); 
            $mail->SMTPSecure = $config["SMTPSecure"];    
            $mail->Port       = $config["port"];      
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($config["from"], "Teknisa Varejo");

            if(empty($cc)) {
                $cc = ["suporte.odhen@teknisa.com"];
            }

            foreach ($recipient as $r) { $mail->addAddress($r); }
            foreach ($cc as $c) { $mail->addCC($c); }
            
            $mail->isHTML(true);                                  
            $mail->Subject = $subject;
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }
    
    private function encrypt_decrypt($action, $string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        // hash
        $key = hash('sha256', $secret_key);
        
        // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
        $iv = substr(hash('sha256', $secret_iv), 0, 16);
        if ( $action == 'encrypt' ) {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        } else if( $action == 'decrypt' ) {
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }
        return $output;
    }
    
    public function getCurrentTimezone($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    public function getMailList() {
        $list = [];

        $sql = "
            SELECT DSEMAILTEC 
              FROM LISTAEMAILTEC
             WHERE CDPROJETO = '004'
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        return $list;
    }
    
    private function decrypt($string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);

        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        return $output;
    } 
    
    public function toFloat($num) {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
	    
	    return  str_replace(".", ",", $tmp);
    }
    
    public function formatNrSeqProdCom($seq) {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }    
    
    public function removeAcentos($texto) {
        $table = array('á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_');
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($texto, $table));
    }
    
    private function resetAccessToken() {
        $sql = "
            UPDATE INTEGRACAO
               SET AUTHTOKEN      = :AUTHTOKEN,
                   AUTHTKNDTCRIAC = :AUTHTKNDTCRIAC
             WHERE IDINTEGRACAO   = 'RAP'
        ";
    	
    	$params = [
    	    ":AUTHTOKEN"      => null,
            ":AUTHTKNDTCRIAC" => null
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
}